<?php

namespace Drupal\permissions_repository\Form;

use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Discovery\YamlDiscovery;
use Drupal\permissions_repository\Service\PermissionsRepositoryFactoryInterface;
use Drupal\permissions_repository\Service\PermissionsRepositoryReturnStyle;
use Drupal\user\RoleStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Override for the built-in user permissions administration form.
 */
class PermissionsRepositoryForm extends FormBase {

  private const NO_SECTION_KEY = '__none__';

  /**
   * The permission handler.
   *
   * @var \Drupal\user\PermissionsRepositoryFactoryInterface
   */
  protected $permissionHandler;

  /**
   * The role storage.
   *
   * @var \Drupal\user\RoleStorageInterface
   */
  protected $roleStorage;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a new UserPermissionsForm.
   *
   * @param \Drupal\user\PermissionsRepositoryFactoryInterface $permission_handler
   *   The permission handler.
   * @param \Drupal\user\RoleStorageInterface $role_storage
   *   The role storage.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Extension\ModuleExtensionList|null $moduleExtensionList
   *   The module extension list.
   */
  public function __construct(PermissionsRepositoryFactoryInterface $permission_handler, RoleStorageInterface $role_storage, ModuleHandlerInterface $module_handler, protected ?ModuleExtensionList $moduleExtensionList = NULL) {
    $this->permissionHandler = $permission_handler;
    $this->roleStorage = $role_storage;
    $this->moduleHandler = $module_handler;
    if ($this->moduleExtensionList === NULL) {
      @trigger_error('Calling ' . __METHOD__ . '() without the $moduleExtensionList argument is deprecated in drupal:10.3.0 and will be required in drupal:12.0.0. See https://www.drupal.org/node/3310017', E_USER_DEPRECATED);
      $this->moduleExtensionList = \Drupal::service('extension.list.module');
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('permissions_repository.repository_factory'),
      $container->get('entity_type.manager')->getStorage('user_role'),
      $container->get('module_handler'),
      $container->get('extension.list.module'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'user_admin_permissions';
  }

  /**
   * Gets the roles to display in this form.
   *
   * @return \Drupal\user\RoleInterface[]
   *   An array of role objects.
   */
  protected function getRoles() {
    return $this->roleStorage->loadMultiple();
  }

  /**
   * Group permissions by the modules that provide them.
   *
   * @return string[][]
   *   A nested array. The outer keys are modules that provide permissions. The
   *   inner arrays are permission names keyed by their machine names.
   */
  protected function permissionsByProvider(): array {
    $yamlDiscovery = new YamlDiscovery('permissions', $this->moduleHandler->getModuleDirectories());

    $list = $yamlDiscovery->findAll();

    $permissionsObj = $this->permissionHandler->createPermissionsRepository($list);

    return $permissionsObj->getAllData(PermissionsRepositoryReturnStyle::SectionKey);

    $permissions = $this->permissionHandler->getPermissions();
    $permissions_by_provider = [];
    foreach ($permissions as $permission_name => $permission) {
      $permissions_by_provider[$permission['provider']][$permission_name] = $permission;
    }
    // Move the access content permission to the Node module if it is installed.
    // @todo Add an alter so that this section can be moved to the Node module.
    if ($this->moduleHandler->moduleExists('node')) {
      // Insert 'access content' before the 'view own unpublished content' key
      // in order to maintain the UI even though the permission is provided by
      // the system module.
      $keys = array_keys($permissions_by_provider['node']);
      $offset = (int) array_search('view own unpublished content', $keys);
      $permissions_by_provider['node'] = array_merge(
        array_slice($permissions_by_provider['node'], 0, $offset),
        ['access content' => $permissions_by_provider['system']['access content']],
        array_slice($permissions_by_provider['node'], $offset)
      );
      unset($permissions_by_provider['system']['access content']);
    }

    return $permissions_by_provider;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $role_names = [];
    $role_permissions = [];
    $admin_roles = [];
    foreach ($this->getRoles() as $role_name => $role) {
      // Retrieve role names for columns.
      $role_names[$role_name] = $role->label();
      // Fetch permissions for the roles.
      $role_permissions[$role_name] = $role->getPermissions();
      $admin_roles[$role_name] = $role->isAdmin();
    }

    // Store $role_names for use when saving the data.
    $form['role_names'] = [
      '#type' => 'value',
      '#value' => $role_names,
    ];
    // Render role/permission overview:
    $hide_descriptions = system_admin_compact_mode();

    $form['system_compact_link'] = [
      '#id' => FALSE,
      '#type' => 'system_compact_link',
    ];

    $form['filters'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['table-filter', 'js-show'],
      ],
    ];

    $form['filters']['text'] = [
      '#type' => 'search',
      '#title' => $this->t('Filter permissions'),
      '#title_display' => 'invisible',
      '#size' => 30,
      '#placeholder' => $this->t('Filter by permission name'),
      '#description' => $this->t('Enter permission name'),
      '#attributes' => [
        'class' => ['table-filter-text'],
        'data-table' => '#permissions',
        'autocomplete' => 'off',
      ],
    ];

    $form['permissions'] = [
      '#type' => 'table',
      '#header' => [$this->t('Permission')],
      '#id' => 'permissions',
      '#attributes' => ['class' => ['permissions', 'js-permissions']],
      '#sticky' => TRUE,
      '#attached' => [
        'library' => [
          'permissions_repository/custom',
        ],
      ],
    ];

    foreach ($role_names as $name) {
      $form['permissions']['#header'][] = [
        'data' => $name,
        'class' => ['checkbox'],
      ];
    }

    foreach ($this->permissionsByProvider() as $provider => $providerData) {
      if (!empty($providerData['main']['extended_help'])) {
        $content['visible_logged_in_only'] = [
          '#markup' => $providerData['main']['extended_help'],
          '#access' => TRUE,
        ];

        $form['permissions'][$provider] = [
          [
            '#type' => 'inline_template',
            '#template' => '{{ module_name }} <a href="" class="question-help" data-tooltip="{{ settings }}">&nbsp;&nbsp;&nbsp;</a>',
            '#context' => [
              'module_name' => $this->moduleExtensionList->getName($provider),
              'settings' => \Drupal\Component\Serialization\Json::encode([
                'content' => \Drupal::service('renderer')->renderInIsolation($content),
                'placement' => 'bottom',
              ]),
            ],
            '#wrapper_attributes' => [
              'colspan' => count($role_names) + 1,
              'class' => ['module'],
              'id' => 'module-' . $provider,
            ],
          ],
        ];
      }
      else {
        $form['permissions'][$provider] = [
          [
            '#wrapper_attributes' => [
              'colspan' => count($role_names) + 1,
              'class' => ['module'],
              'id' => 'module-' . $provider,
            ],
            '#markup' => $this->moduleExtensionList->getName($provider),
          ],
        ];
      }

      $sectionToPermissions = $this->dividePermissionsIntoSections($providerData);
      foreach ($sectionToPermissions as $sectionKey => $permissions) {
        if ($sectionKey !== self::NO_SECTION_KEY) {
          $form['permissions'][$sectionKey] = [
            [
              '#wrapper_attributes' => [
                'colspan' => count($role_names) + 1,
                'class' => ['module'],
                'id' => 'module-sectionHeader-' . $provider . '-' . $sectionKey,
              ],
              '#markup' => '&nbsp;&nbsp;&nbsp;&nbsp;' . $providerData['sections'][$sectionKey]['title'],
            ],
          ];
        }

        foreach ($permissions as $permissionKey => $permissionAry) {
          //$this->addPermissions($permissionKey, $permissionAry
          // Fill in default values for the permission.
          $permissionAry += [
            'description' => '',
            'restrict access' => FALSE,
            'warning' => !empty($permissionAry['restrict access']) ? $this->t('Warning: Give to trusted roles only; this permission has security implications.') : '',
          ];
          $form['permissions'][$permissionKey]['description'] = [
            '#type' => 'inline_template',
            '#template' => '<div class="permission"><span class="title table-filter-text-source">{{ title }}</span>{% if description or warning %}<div class="description">{% if warning %}<em class="permission-warning">{{ warning }}</em> {% endif %}{{ description }}</div>{% endif %}</div>',
            '#context' => [
              'title' => $permissionAry['title'],
            ],
          ];
          // Show the permission description.
          if (!$hide_descriptions) {
            $form['permissions'][$permissionKey]['description']['#context']['description'] = $permissionAry['description'];
            $form['permissions'][$permissionKey]['description']['#context']['warning'] = $permissionAry['warning'];
          }
          foreach ($role_names as $rid => $name) {
            $form['permissions'][$permissionKey][$rid] = [
              '#title' => $name . ': ' . $permissionAry['title'],
              '#title_display' => 'invisible',
              '#wrapper_attributes' => [
                'class' => ['checkbox'],
              ],
              '#type' => 'checkbox',
              '#default_value' => in_array($permissionKey, $role_permissions[$rid]) ? 1 : 0,
              '#attributes' => ['class' => ['rid-' . $rid, 'js-rid-' . $rid]],
              '#parents' => [$rid, $permissionKey],
            ];
            // Show a column of disabled but checked checkboxes.
            if ($admin_roles[$rid]) {
              $form['permissions'][$permissionKey][$rid]['#disabled'] = TRUE;
              $form['permissions'][$permissionKey][$rid]['#default_value'] = TRUE;
            }
          }
        }
      }
    }

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save permissions'),
      '#button_type' => 'primary',
    ];

    $form['#attached']['library'][] = 'user/drupal.user.permissions';
    $form['#attached']['library'][] = 'tooltip/tooltip';

    return $form;
  }

  protected function dividePermissionsIntoSections($providerData) {
    $ret = [
      self::NO_SECTION_KEY => [],
    ];

    if (empty($providerData['permissions'])) {
      return $ret;
    }

    foreach ($providerData['permissions'] as $permissionsKey => $permissionsAry) {
      $permissionAdded = FALSE;
      if (!empty($permissionsAry['section'])) {
        $sectionKey = $permissionsAry['section'];
        if (!empty($providerData['sections'][$sectionKey])) {
          $ret[$sectionKey][$permissionsKey] = $permissionsAry;
          $permissionAdded = TRUE;
        }
      }

      if (!$permissionAdded) {
        $ret[self::NO_SECTION_KEY][$permissionsKey] = $permissionsAry;
      }
    }

    return $ret;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach ($form_state->getValue('role_names') as $role_name => $name) {
      user_role_change_permissions($role_name, (array) $form_state->getValue($role_name));
    }

    $this->messenger()->addStatus($this->t('The changes have been saved.'));
  }

}
